@@ -24,11 +24,9 @@
 #ifndef GLPIOS_H
 #define GLPIOS_H
 
-#define GLP_TREE
 typedef struct glp_tree glp_tree;
 
 #include "glpapi.h"
-#include "glpscg.h"
 
 typedef struct IOSLOT IOSLOT;
 typedef struct IOSNPD IOSNPD;
@@ -38,12 +36,13 @@ typedef struct IOSROW IOSROW;
 typedef struct IOSAIJ IOSAIJ;
 typedef struct IOSPOOL IOSPOOL;
 typedef struct IOSCUT IOSCUT;
-typedef struct IOSRIB IOSRIB;
 
 struct glp_tree
 {     /* branch-and-bound tree */
+      int magic;
+      /* magic value used for debugging */
       DMP *pool;
-      /* memory pool to store IOS components */
+      /* memory pool to store all IOS components */
       int n;
       /* number of columns (variables) */
       /*--------------------------------------------------------------*/
@@ -52,13 +51,13 @@ struct glp_tree
          exit from the solver) */
       int orig_m;
       /* number of rows */
-      int *orig_type; /* int orig_type[1+orig_m+n]; */
+      unsigned char *orig_type; /* uchar orig_type[1+orig_m+n]; */
       /* types of all variables */
       double *orig_lb; /* double orig_lb[1+orig_m+n]; */
       /* lower bounds of all variables */
       double *orig_ub; /* double orig_ub[1+orig_m+n]; */
       /* upper bounds of all variables */
-      int *orig_stat; /* int orig_stat[1+orig_m+n]; */
+      unsigned char *orig_stat; /* uchar orig_stat[1+orig_m+n]; */
       /* statuses of all variables */
       double *orig_prim; /* double orig_prim[1+orig_m+n]; */
       /* primal values of all variables */
@@ -101,13 +100,13 @@ struct glp_tree
       /* problem components corresponding to the root subproblem */
       int root_m;
       /* number of rows */
-      int *root_type; /* int root_type[1+root_m+n]; */
+      unsigned char *root_type; /* uchar root_type[1+root_m+n]; */
       /* types of all variables */
       double *root_lb; /* double root_lb[1+root_m+n]; */
       /* lower bounds of all variables */
       double *root_ub; /* double root_ub[1+root_m+n]; */
       /* upper bounds of all variables */
-      int *root_stat; /* int root_stat[1+root_m+n]; */
+      unsigned char *root_stat; /* uchar root_stat[1+root_m+n]; */
       /* statuses of all variables */
       /*--------------------------------------------------------------*/
       /* current subproblem and its LP relaxation */
@@ -122,67 +121,38 @@ struct glp_tree
          subproblem (note that the root subproblem may differ from the
          original MIP, because it may be preprocessed and/or may have
          additional rows) */
-      int solved;
-      /* this count shows how many times LP relaxation of the current
-         subproblem is solved */
-      int *non_int; /* int non_int[1+n]; */
+      unsigned char *non_int; /* uchar non_int[1+n]; */
       /* these column flags are set each time when LP relaxation of the
-         current subproblem is solved;
+         current subproblem has been solved;
          non_int[0] is not used;
          non_int[j], 1 <= j <= n, is j-th column flag; if this flag is
          set, corresponding variable is required to be integer, but its
          value in basic solution is fractional */
       /*--------------------------------------------------------------*/
       /* problem components corresponding to the parent (predecessor)
-         subproblem for the current subproblem (used to inspect changes
-         on freezing the current subproblem) */
+         subproblem for the current subproblem; used to inspect changes
+         on freezing the current subproblem */
       int pred_m;
       /* number of rows */
       int pred_max;
       /* length of the following four arrays (enlarged automatically),
          pred_max >= pred_m + n */
-      int *pred_type; /* int pred_type[1+pred_m+n]; */
+      unsigned char *pred_type; /* uchar pred_type[1+pred_m+n]; */
       /* types of all variables */
       double *pred_lb; /* double pred_lb[1+pred_m+n]; */
       /* lower bounds of all variables */
       double *pred_ub; /* double pred_ub[1+pred_m+n]; */
       /* upper bounds of all variables */
-      int *pred_stat; /* int pred_stat[1+pred_m+n]; */
+      unsigned char *pred_stat; /* uchar pred_stat[1+pred_m+n]; */
       /* statuses of all variables */
-      /*--------------------------------------------------------------*/
-      /* cut generator (under construction) */
+      /****************************************************************/
+      /* built-in cut generators segment */
       IOSPOOL *local;
       /* local cut pool */
-      int first_attempt;
-      int max_added_cuts;
-      double min_eff;
-      int miss;
-      int just_selected;
       void *mir_gen;
       /* pointer to working area used by the MIR cut generator */
       void *clq_gen;
       /* pointer to working area used by the clique cut generator */
-      int round;
-      /* round number; shows how many times the cut generating routine
-         is called for the same subproblem; 1 means call for the first
-         time */
-      /*--------------------------------------------------------------*/
-      /* conflict graph for the current subproblem, or, if it does not
-         exists, for the root subproblem */
-      int *n_ref; /* int n_ref[1+n]; */
-      /* n_ref[j], 1 <= j <= n, is the number of a node corresponding
-         to binary variable x[j]; n_ref[j] = 0 means that the node is
-         not assigned yet */
-      int *c_ref; /* int c_ref[1+n]; */
-      /* c_ref[j], 1 <= j <= n, is the number of a node corresponding
-         to complemented binary variable x'[j]; c_ref[j] = 0 means that
-         the node is not assigned yet */
-      SCG *g;
-      /* conflict graph for the current or root subproblem */
-      int *j_ref; /* int j_ref[1+g->n_max]; */
-      /* j_ref[k], 1 <= k <= g->n, is the number of a binary variable,
-         which refers to node k, that is, j_ref[k] = j if n_ref[j] = k
-         or c_ref[j] = k */
       /*--------------------------------------------------------------*/
       void *pcost;
       /* pointer to working area used on pseudocost branching */
@@ -207,27 +177,29 @@ struct glp_tree
       int reason;
       /* flag indicating the reason why the callback routine is being
          called (see glpk.h) */
+      int stop;
+      /* flag indicating that the callback routine requires premature
+         termination of the search */
+      int next_p;
+      /* reference number of active subproblem selected to continue
+         the search; 0 means no subproblem has been selected */
       int reopt;
       /* flag indicating that the current LP relaxation needs to be
          re-optimized */
       int reinv;
       /* flag indicating that some (non-active) rows were removed from
          the current LP relaxation, so if there no new rows appear, the
-         basis must be warmed up */
+         basis must be re-factorized */
       int br_var;
       /* the number of variable chosen to branch on */
       int br_sel;
-      /* flag indicating which branch (subproblem) should be selected
-         to continue the search:
+      /* flag indicating which branch (subproblem) is suggested to be
+         selected to continue the search:
          GLP_DN_BRNCH - select down-branch
          GLP_UP_BRNCH - select up-branch
          GLP_NO_BRNCH - use general selection technique */
-      IOSNPD *btrack;
-      /* pointer to active subproblem selected to continue the search;
-         NULL means no subproblem has been selected */
-      int terminate;
-      /* flag indicating that the callback routine requires premature
-         termination of the search */
+      int child;
+      /* subproblem reference number corresponding to br_sel */
 };
 
 struct IOSLOT
@@ -252,31 +224,21 @@ struct IOSNPD
       /* if count = 0, this subproblem is active; if count > 0, this
          subproblem is inactive, in which case count is the number of
          its child subproblems */
+      /* the following three linked lists are destroyed on reviving and
+         built anew on freezing the subproblem: */
       IOSBND *b_ptr;
       /* linked list of rows and columns of the parent subproblem whose
-         types and bounds were changed;
-         this list is destroyed on reviving and built anew on freezing
-         the subproblem */
+         types and bounds were changed */
       IOSTAT *s_ptr;
       /* linked list of rows and columns of the parent subproblem whose
-         statuses were changed;
-         this list is destroyed on reviving and built anew on freezing
-         the subproblem */
+         statuses were changed */
       IOSROW *r_ptr;
-      /* linked list of rows (cuts) added to the parent subproblem;
-         this list is destroyed on reviving and built anew on freezing
-         the subproblem */
-      /*--------------------------------------------------------------*/
-      /* changes in the conflict graph for this subproblem */
-      int own_nn;
-      /* number of nodes added for this subproblem */
-      int own_nc;
-      /* number of cliques added for this subproblem */
-      IOSRIB *e_ptr;
-      /* linked list of new edges added for this subproblem, where both
-         endpoints were added in some ancestor subproblem */
-      /*--------------------------------------------------------------*/
-#if 1 /* 04/X-2008 */
+      /* linked list of rows (cuts) added to the parent subproblem */
+      int solved;
+      /* how many times LP relaxation of this subproblem was solved;
+         for inactive subproblem this count is always non-zero;
+         for active subproblem, which is not current, this count may be
+         non-zero, if the subproblem was temporarily suspended */
       double lp_obj;
       /* optimal objective value to LP relaxation of this subproblem;
          on creating a subproblem this value is inherited from its
@@ -284,7 +246,6 @@ struct IOSNPD
          value is initially set to -DBL_MAX (minimization) or +DBL_MAX
          (maximization); each time the subproblem is re-optimized, this
          value is appropriately changed */
-#endif
       double bound;
       /* local lower (minimization) or upper (maximization) bound for
          integer optimal solution to *this* subproblem; this bound is
@@ -295,7 +256,18 @@ struct IOSNPD
          root subproblem its local bound is initially set to -DBL_MAX
          (minimization) or +DBL_MAX (maximization) and then improved as
          the root LP relaxation has been solved */
-#if 1 /* 04/X-2008 */
+      /* the following two quantities are defined only if LP relaxation
+         of this subproblem was solved at least once (solved > 0): */
+      int ii_cnt;
+      /* number of integer variables whose value in optimal solution to
+         LP relaxation of this subproblem is fractional */
+      double ii_sum;
+      /* sum of integer infeasibilities */
+#if 1 /* 30/XI-2009 */
+      int changed;
+      /* how many times this subproblem was re-formulated (by adding
+         cutting plane constraints) */
+#endif
       int br_var;
       /* ordinal number of branching variable, 1 <= br_var <= n, used
          to split this subproblem; 0 means that either this subproblem
@@ -303,19 +275,10 @@ struct IOSNPD
       double br_val;
       /* (fractional) value of branching variable in optimal solution
          to final LP relaxation of this subproblem */
-#endif
-      /* if this subproblem is inactive, the following two quantities
-         correspond to final optimal solution of its LP relaxation; for
-         active subproblems these quantities are undefined */
-      int ii_cnt;
-      /* number of columns (structural variables) of integer kind whose
-         primal values are fractional */
-      double ii_sum;
-      /* the sum of integer infeasibilities */
       void *data; /* char data[tree->cb_size]; */
       /* pointer to the application-specific data */
       IOSNPD *temp;
-      /* auxiliary pointer used by some routines */
+      /* working pointer used by some routines */
       IOSNPD *prev;
       /* pointer to previous subproblem in the active list */
       IOSNPD *next;
@@ -328,7 +291,7 @@ struct IOSBND
       /* ordinal number of corresponding row (1 <= k <= m) or column
          (m+1 <= k <= m+n), where m and n are the number of rows and
          columns, resp., in the parent subproblem */
-      int type;
+      unsigned char type;
       /* new type */
       double lb;
       /* new lower bound */
@@ -344,32 +307,32 @@ struct IOSTAT
       /* ordinal number of corresponding row (1 <= k <= m) or column
          (m+1 <= k <= m+n), where m and n are the number of rows and
          columns, resp., in the parent subproblem */
-      int stat;
+      unsigned char stat;
       /* new status */
       IOSTAT *next;
       /* pointer to next entry for the same subproblem */
 };
 
 struct IOSROW
-{     /* row (cut) addition entry */
+{     /* row (constraint) addition entry */
       char *name;
       /* row name or NULL */
-#if 1 /* 20/IX-2008 */
       unsigned char origin;
+      /* row origin flag (see glp_attr.origin) */
       unsigned char klass;
-#endif
-      int type;
-      /* row type */
+      /* row class descriptor (see glp_attr.klass) */
+      unsigned char type;
+      /* row type (GLP_LO, GLP_UP, etc.) */
       double lb;
       /* row lower bound */
       double ub;
       /* row upper bound */
       IOSAIJ *ptr;
-      /* pointer to the coefficient list */
+      /* pointer to the row coefficient list */
       double rii;
       /* row scale factor */
-      int stat;
-      /* row status */
+      unsigned char stat;
+      /* row status (GLP_BS, GLP_NL, etc.) */
       IOSROW *next;
       /* pointer to next entry for the same subproblem */
 };
@@ -377,9 +340,9 @@ struct IOSROW
 struct IOSAIJ
 {     /* constraint coefficient */
       int j;
-      /* column number */
+      /* variable (column) number, 1 <= j <= n */
       double val;
-      /* coefficient value */
+      /* non-zero coefficient value */
       IOSAIJ *next;
       /* pointer to next coefficient for the same row */
 };
@@ -389,47 +352,34 @@ struct IOSPOOL
       int size;
       /* pool size = number of cuts in the pool */
       IOSCUT *head;
-      /* pointer to the first cut row */
+      /* pointer to the first cut */
       IOSCUT *tail;
-      /* pointer to the last cut row */
+      /* pointer to the last cut */
       int ord;
-      /* ordinal number of the current cut row, 1 <= ord <= size */
+      /* ordinal number of the current cut, 1 <= ord <= size */
       IOSCUT *curr;
-      /* pointer to the current cut row */
+      /* pointer to the current cut */
 };
 
 struct IOSCUT
-{     /* cut row (cutting plane constraint) */
-#if 1 /* 20/IX-2008 */
+{     /* cut (cutting plane constraint) */
       char *name;
+      /* cut name or NULL */
       unsigned char klass;
-#endif
+      /* cut class descriptor (see glp_attr.klass) */
       IOSAIJ *ptr;
-      /* pointer to the list of row coefficients */
-      int type;
-      /* row type:
+      /* pointer to the cut coefficient list */
+      unsigned char type;
+      /* cut type:
          GLP_LO: sum a[j] * x[j] >= b
          GLP_UP: sum a[j] * x[j] <= b
          GLP_FX: sum a[j] * x[j]  = b */
       double rhs;
-      /* row right-hand side */
+      /* cut right-hand side */
       IOSCUT *prev;
-      /* pointer to the previous cut row */
+      /* pointer to previous cut */
       IOSCUT *next;
-      /* pointer to the next cut row */
-};
-
-struct IOSRIB
-{     /* conflict edge addition entry */
-      int j1;
-      /* first variable number (j1 < 0 means complement) */
-      int j2;
-      /* second variable number (j2 < 0 means complement) */
-      SCGRIB *e;
-      /* pointer to the edge descriptor (NULL means the edge is missing
-         in the current graph) */
-      IOSRIB *next;
-      /* pointer to next entry for the same subproblem */
+      /* pointer to next cut */
 };
 
 #define ios_create_tree _glp_ios_create_tree
@@ -506,17 +456,10 @@ void ios_clear_pool(glp_tree *tree, IOSPOOL *pool);
 void ios_delete_pool(glp_tree *tree, IOSPOOL *pool);
 /* delete cut pool */
 
-#define ios_add_edge _glp_ios_add_edge
-void ios_add_edge(glp_tree *tree, int j1, int j2);
-/* add new edge to the conflict graph */
-
 #define ios_preprocess_node _glp_ios_preprocess_node
 int ios_preprocess_node(glp_tree *tree, int max_pass);
 /* preprocess current subproblem */
 
-#define ios_mostf_branch _glp_ios_mostf_branch
-void ios_mostf_branch(glp_tree *T);
-
 #define ios_driver _glp_ios_driver
 int ios_driver(glp_tree *tree);
 /* branch-and-bound driver */
@@ -616,7 +559,7 @@ void *ios_pcost_init(glp_tree *tree);
 /* initialize working data used on pseudocost branching */
 
 #define ios_pcost_branch _glp_ios_pcost_branch
-void ios_pcost_branch(glp_tree *tree);
+int ios_pcost_branch(glp_tree *T, int *next);
 /* choose branching variable with pseudocost branching */
 
 #define ios_pcost_update _glp_ios_pcost_update
@@ -631,6 +574,18 @@ void ios_pcost_free(glp_tree *tree);
 void ios_feas_pump(glp_tree *T);
 /* feasibility pump heuristic */
 
+#define ios_process_cuts _glp_ios_process_cuts
+void ios_process_cuts(glp_tree *T);
+/* process cuts stored in the local cut pool */
+
+#define ios_choose_node _glp_ios_choose_node
+int ios_choose_node(glp_tree *T);
+/* select subproblem to continue the search */
+
+#define ios_choose_var _glp_ios_choose_var
+int ios_choose_var(glp_tree *T, int *next);
+/* select variable to branch on */
+
 #endif
 
 /* eof */
