@@ -22,10 +22,146 @@
 ***********************************************************************/
 
 #include "glpapi.h"
-#include "glplib.h"
-#if 0 /* 13/IV-2009 */
-#include "glpmps.h"
-#endif
+
+struct LPXCPS
+{     /* control parameters and statistics */
+      int msg_lev;
+      /* level of messages output by the solver:
+         0 - no output
+         1 - error messages only
+         2 - normal output
+         3 - full output (includes informational messages) */
+      int scale;
+      /* scaling option:
+         0 - no scaling
+         1 - equilibration scaling
+         2 - geometric mean scaling
+         3 - geometric mean scaling, then equilibration scaling */
+      int dual;
+      /* dual simplex option:
+         0 - use primal simplex
+         1 - use dual simplex */
+      int price;
+      /* pricing option (for both primal and dual simplex):
+         0 - textbook pricing
+         1 - steepest edge pricing */
+      double relax;
+      /* relaxation parameter used in the ratio test; if it is zero,
+         the textbook ratio test is used; if it is non-zero (should be
+         positive), Harris' two-pass ratio test is used; in the latter
+         case on the first pass basic variables (in the case of primal
+         simplex) or reduced costs of non-basic variables (in the case
+         of dual simplex) are allowed to slightly violate their bounds,
+         but not more than (relax * tol_bnd) or (relax * tol_dj) (thus,
+         relax is a percentage of tol_bnd or tol_dj) */
+      double tol_bnd;
+      /* relative tolerance used to check if the current basic solution
+         is primal feasible */
+      double tol_dj;
+      /* absolute tolerance used to check if the current basic solution
+         is dual feasible */
+      double tol_piv;
+      /* relative tolerance used to choose eligible pivotal elements of
+         the simplex table in the ratio test */
+      int round;
+      /* solution rounding option:
+         0 - report all computed values and reduced costs "as is"
+         1 - if possible (allowed by the tolerances), replace computed
+             values and reduced costs which are close to zero by exact
+             zeros */
+      double obj_ll;
+      /* lower limit of the objective function; if on the phase II the
+         objective function reaches this limit and continues decreasing,
+         the solver stops the search */
+      double obj_ul;
+      /* upper limit of the objective function; if on the phase II the
+         objective function reaches this limit and continues increasing,
+         the solver stops the search */
+      int it_lim;
+      /* simplex iterations limit; if this value is positive, it is
+         decreased by one each time when one simplex iteration has been
+         performed, and reaching zero value signals the solver to stop
+         the search; negative value means no iterations limit */
+      double tm_lim;
+      /* searching time limit, in seconds; if this value is positive,
+         it is decreased each time when one simplex iteration has been
+         performed by the amount of time spent for the iteration, and
+         reaching zero value signals the solver to stop the search;
+         negative value means no time limit */
+      int out_frq;
+      /* output frequency, in iterations; this parameter specifies how
+         frequently the solver sends information about the solution to
+         the standard output */
+      double out_dly;
+      /* output delay, in seconds; this parameter specifies how long
+         the solver should delay sending information about the solution
+         to the standard output; zero value means no delay */
+      int branch; /* MIP */
+      /* branching heuristic:
+         0 - branch on first variable
+         1 - branch on last variable
+         2 - branch using heuristic by Driebeck and Tomlin
+         3 - branch on most fractional variable */
+      int btrack; /* MIP */
+      /* backtracking heuristic:
+         0 - select most recent node (depth first search)
+         1 - select earliest node (breadth first search)
+         2 - select node using the best projection heuristic
+         3 - select node with best local bound */
+      double tol_int; /* MIP */
+      /* absolute tolerance used to check if the current basic solution
+         is integer feasible */
+      double tol_obj; /* MIP */
+      /* relative tolerance used to check if the value of the objective
+         function is not better than in the best known integer feasible
+         solution */
+      int mps_info; /* lpx_write_mps */
+      /* if this flag is set, the routine lpx_write_mps outputs several
+         comment cards that contains some information about the problem;
+         otherwise the routine outputs no comment cards */
+      int mps_obj; /* lpx_write_mps */
+      /* this parameter tells the routine lpx_write_mps how to output
+         the objective function row:
+         0 - never output objective function row
+         1 - always output objective function row
+         2 - output objective function row if and only if the problem
+             has no free rows */
+      int mps_orig; /* lpx_write_mps */
+      /* if this flag is set, the routine lpx_write_mps uses original
+         row and column symbolic names; otherwise the routine generates
+         plain names using ordinal numbers of rows and columns */
+      int mps_wide; /* lpx_write_mps */
+      /* if this flag is set, the routine lpx_write_mps uses all data
+         fields; otherwise the routine keeps fields 5 and 6 empty */
+      int mps_free; /* lpx_write_mps */
+      /* if this flag is set, the routine lpx_write_mps omits column
+         and vector names everytime if possible (free style); otherwise
+         the routine never omits these names (pedantic style) */
+      int mps_skip; /* lpx_write_mps */
+      /* if this flag is set, the routine lpx_write_mps skips empty
+         columns (i.e. which has no constraint coefficients); otherwise
+         the routine outputs all columns */
+      int lpt_orig; /* lpx_write_lpt */
+      /* if this flag is set, the routine lpx_write_lpt uses original
+         row and column symbolic names; otherwise the routine generates
+         plain names using ordinal numbers of rows and columns */
+      int presol; /* lpx_simplex */
+      /* LP presolver option:
+         0 - do not use LP presolver
+         1 - use LP presolver */
+      int binarize; /* lpx_intopt */
+      /* if this flag is set, the routine lpx_intopt replaces integer
+         columns by binary ones */
+      int use_cuts; /* lpx_intopt */
+      /* if this flag is set, the routine lpx_intopt tries generating
+         cutting planes:
+         LPX_C_COVER  - mixed cover cuts
+         LPX_C_CLIQUE - clique cuts
+         LPX_C_GOMORY - Gomory's mixed integer cuts
+         LPX_C_ALL    - all cuts */
+      double mip_gap; /* MIP */
+      /* relative MIP gap tolerance */
+};
 
 LPX *lpx_create_prob(void)
 {     /* create problem object */
@@ -601,6 +737,34 @@ int lpx_eval_tab_col(LPX *lp, int k, int ind[], double val[])
       return glp_eval_tab_col(lp, k, ind, val);
 }
 
+int lpx_transform_row(LPX *lp, int len, int ind[], double val[])
+{     /* transform explicitly specified row */
+      return glp_transform_row(lp, len, ind, val);
+}
+
+int lpx_transform_col(LPX *lp, int len, int ind[], double val[])
+{     /* transform explicitly specified column */
+      return glp_transform_col(lp, len, ind, val);
+}
+
+int lpx_prim_ratio_test(LPX *lp, int len, const int ind[],
+      const double val[], int how, double tol)
+{     /* perform primal ratio test */
+      int piv;
+      piv = glp_prim_rtest(lp, len, ind, val, how, tol);
+      xassert(0 <= piv && piv <= len);
+      return piv == 0 ? 0 : ind[piv];
+}
+
+int lpx_dual_ratio_test(LPX *lp, int len, const int ind[],
+      const double val[], int how, double tol)
+{     /* perform dual ratio test */
+      int piv;
+      piv = glp_dual_rtest(lp, len, ind, val, how, tol);
+      xassert(0 <= piv && piv <= len);
+      return piv == 0 ? 0 : ind[piv];
+}
+
 int lpx_interior(LPX *lp)
 {     /* easy-to-use driver to the interior-point method */
       int ret;
@@ -828,9 +992,11 @@ void lpx_check_int(LPX *lp, LPXKKT *kkt)
       return;
 }
 
-void lpx_reset_parms(LPX *lp)
+#if 1 /* 17/XI-2009 */
+static void reset_parms(LPX *lp)
 {     /* reset control parameters to default values */
-      struct LPXCPS *cps = lp->cps;
+      struct LPXCPS *cps = lp->parms;
+      xassert(cps != NULL);
       cps->msg_lev  = 3;
       cps->scale    = 1;
       cps->dual     = 0;
@@ -864,10 +1030,35 @@ void lpx_reset_parms(LPX *lp)
       cps->mip_gap = 0.0;
       return;
 }
+#endif
+
+#if 1 /* 17/XI-2009 */
+static struct LPXCPS *access_parms(LPX *lp)
+{     /* allocate and initialize control parameters, if necessary */
+      if (lp->parms == NULL)
+      {  lp->parms = xmalloc(sizeof(struct LPXCPS));
+         reset_parms(lp);
+      }
+      return lp->parms;
+}
+#endif
+
+#if 1 /* 17/XI-2009 */
+void lpx_reset_parms(LPX *lp)
+{     /* reset control parameters to default values */
+      access_parms(lp);
+      reset_parms(lp);
+      return;
+}
+#endif
 
 void lpx_set_int_parm(LPX *lp, int parm, int val)
 {     /* set (change) integer control parameter */
+#if 0 /* 17/XI-2009 */
       struct LPXCPS *cps = lp->cps;
+#else
+      struct LPXCPS *cps = access_parms(lp);
+#endif
       switch (parm)
       {  case LPX_K_MSGLEV:
             if (!(0 <= val && val <= 3))
@@ -1016,7 +1207,11 @@ void lpx_set_int_parm(LPX *lp, int parm, int val)
 
 int lpx_get_int_parm(LPX *lp, int parm)
 {     /* query integer control parameter */
+#if 0 /* 17/XI-2009 */
       struct LPXCPS *cps = lp->cps;
+#else
+      struct LPXCPS *cps = access_parms(lp);
+#endif
       int val = 0;
       switch (parm)
       {  case LPX_K_MSGLEV:
@@ -1087,7 +1282,11 @@ int lpx_get_int_parm(LPX *lp, int parm)
 
 void lpx_set_real_parm(LPX *lp, int parm, double val)
 {     /* set (change) real control parameter */
+#if 0 /* 17/XI-2009 */
       struct LPXCPS *cps = lp->cps;
+#else
+      struct LPXCPS *cps = access_parms(lp);
+#endif
       switch (parm)
       {  case LPX_K_RELAX:
             if (!(0.0 <= val && val <= 1.0))
@@ -1166,7 +1365,11 @@ void lpx_set_real_parm(LPX *lp, int parm, double val)
 
 double lpx_get_real_parm(LPX *lp, int parm)
 {     /* query real control parameter */
+#if 0 /* 17/XI-2009 */
       struct LPXCPS *cps = lp->cps;
+#else
+      struct LPXCPS *cps = access_parms(lp);
+#endif
       double val = 0.0;
       switch (parm)
       {  case LPX_K_RELAX:
